// Copyright (c) 2016, SolutionERP LLC and contributors
// For license information, please see license.txt
/* eslint-disable */

function get_filters() {
	let filters = [
		{
			"fieldname":"company",
			"label": __("Company"),
			"fieldtype": "Link",
			"options": "Company",
			"default": xhiveframework.defaults.get_user_default("Company"),
			"reqd": 1
		},
		{
			"fieldname":"filter_based_on",
			"label": __("Filter Based On"),
			"fieldtype": "Select",
			"options": ["Fiscal Year", "Date Range"],
			"default": ["Fiscal Year"],
			"reqd": 1,
			on_change: function() {
				let filter_based_on = xhiveframework.query_report.get_filter_value('filter_based_on');
				xhiveframework.query_report.toggle_filter_display('from_fiscal_year', filter_based_on === 'Date Range');
				xhiveframework.query_report.toggle_filter_display('to_fiscal_year', filter_based_on === 'Date Range');
				xhiveframework.query_report.toggle_filter_display('period_start_date', filter_based_on === 'Fiscal Year');
				xhiveframework.query_report.toggle_filter_display('period_end_date', filter_based_on === 'Fiscal Year');

				xhiveframework.query_report.refresh();
			}
		},
		{
			"fieldname":"period_start_date",
			"label": __("Start Date"),
			"fieldtype": "Date",
			"hidden": 1,
			"reqd": 1
		},
		{
			"fieldname":"period_end_date",
			"label": __("End Date"),
			"fieldtype": "Date",
			"hidden": 1,
			"reqd": 1
		},
		{
			"fieldname":"from_fiscal_year",
			"label": __("Start Year"),
			"fieldtype": "Link",
			"options": "Fiscal Year",
			"default": xhiveframework.defaults.get_user_default("fiscal_year"),
			"reqd": 1
		},
		{
			"fieldname":"to_fiscal_year",
			"label": __("End Year"),
			"fieldtype": "Link",
			"options": "Fiscal Year",
			"default": xhiveframework.defaults.get_user_default("fiscal_year"),
			"reqd": 1
		},
		{
			"fieldname": "periodicity",
			"label": __("Periodicity"),
			"fieldtype": "Select",
			"options": [
				{ "value": "Monthly", "label": __("Monthly") },
				{ "value": "Quarterly", "label": __("Quarterly") },
				{ "value": "Half-Yearly", "label": __("Half-Yearly") },
				{ "value": "Yearly", "label": __("Yearly") }
			],
			"default": "Monthly",
			"reqd": 1
		},
		{
			"fieldname": "type",
			"label": __("Invoice Type"),
			"fieldtype": "Select",
			"options": [
				{ "value": "Revenue", "label": __("Revenue") },
				{ "value": "Expense", "label": __("Expense") }
			],
			"default": "Revenue",
			"reqd": 1
		},
		{
			"fieldname" : "with_upcoming_postings",
			"label": __("Show with upcoming revenue/expense"),
			"fieldtype": "Check",
			"default": 1
		}
	]

	return filters;
}

xhiveframework.query_reports["Deferred Revenue and Expense"] = {
	"filters": get_filters(),
	"formatter": function(value, row, column, data, default_formatter){
		return default_formatter(value, row, column, data);
	},
	onload: function(report){
		let fiscal_year = xhiveframework.defaults.get_user_default("fiscal_year");

		xhiveframework.model.with_doc("Fiscal Year", fiscal_year, function(r) {
			var fy = xhiveframework.model.get_doc("Fiscal Year", fiscal_year);
			xhiveframework.query_report.set_filter_value({
				period_start_date: fy.year_start_date,
				period_end_date: fy.year_end_date
			});
		});
	}
};

